# Get content from the config.json
$config = Get-Content -Path .\config.json | ConvertFrom-Json

$xu_location = $config.xu_location
$xu_server = $config.xu_server
$xu_port = $config.xu_port
$end_of_upload_api = $config.end_of_upload_api

$extraction_start_date = $config.extraction_start_date
$extraction_end_date = $config.extraction_end_date
$language = $config.language
$exchange_rate_type = $config.exchange_rate_type

# Create a new log file for every run
$logFile = ".\extraction.log"

if (Test-Path -Path .\$logFile -PathType Leaf) {
    Remove-Item .\$logFile
}

# Function for appending text to the log file
function WriteLog
{
Param ([string]$logText)
$timestamp = (Get-Date).toString("dd/MM/yyyy HH:mm:ss")
Add-content $logFile -value "$timestamp $logText"
Write-Output $logText
}

# Function for checking the error code of extractions
function CheckExtractionError
{
Param ([string]$extraction)
if ($LASTEXITCODE -ne 0) {
    WriteLog("$extraction failed with return code: $LASTEXITCODE")
    Set-Variable -Name "extraction_failed" -Value 1 -Scope global
}
}

# If at least one extraction fails, do not upload to blob storage.
Set-Variable -Name "extraction_failed" -Value 0 -Scope global

# Extraction commands
# Transactional tables
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_BKPF" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('P2P_BKPF')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_BSAK" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('P2P_BSAK')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_BSE_CLR" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('P2P_BSE_CLR')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_BSIK" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('P2P_BSIK')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_CDHDR" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('P2P_CDHDR')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_CDPOS" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('P2P_CDPOS')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_EBAN" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('P2P_EBAN')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_EKBE" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('P2P_EKBE')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_EKET"
CheckExtractionError('P2P_EKET')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_EKKN"
CheckExtractionError('P2P_EKKN')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_EKKO" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('P2P_EKKO')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_EKPO" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('P2P_EKPO')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_RBKP" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('P2P_RBKP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_RSEG" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('P2P_RSEG')

# Master data tables 
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_ADCP"
CheckExtractionError('P2P_ADCP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_ADRP"
CheckExtractionError('P2P_ADRP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_CSKT" -o "language=$language"
CheckExtractionError('P2P_CSKT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_KNA1"
CheckExtractionError('P2P_KNA1')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_LFA1"
CheckExtractionError('P2P_LFA1')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_MAKT" -o "language=$language"
CheckExtractionError('P2P_MAKT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_T001"
CheckExtractionError('P2P_T001')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_T001L"
CheckExtractionError('P2P_T001L')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_T001W"
CheckExtractionError('P2P_T001W')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_T005T" -o "language=$language"
CheckExtractionError('P2P_T005T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_T005U" -o "language=$language"
CheckExtractionError('P2P_T005U')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_T006A" -o "language=$language"
CheckExtractionError('P2P_T006A')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_T008T" -o "language=$language"
CheckExtractionError('P2P_T008T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_T023T" -o "language=$language"
CheckExtractionError('P2P_T023T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_T024"
CheckExtractionError('P2P_T024')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_T024E"
CheckExtractionError('P2P_T024E')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_T161T" -o "language=$language"
CheckExtractionError('P2P_T161T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_TCURF" -o "exchange_rate_type=$exchange_rate_type"
CheckExtractionError('P2P_TCURF')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_TCURR" -o "exchange_rate_type=$exchange_rate_type"
CheckExtractionError('P2P_TCURR')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_TCURX"
CheckExtractionError('P2P_TCURX')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_TGSBT" -o "language=$language"
CheckExtractionError('P2P_TGSBT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_USR02"
CheckExtractionError('P2P_USR02')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "P2P_USR21"
CheckExtractionError('P2P_USR21')

# Only ingest data if extraction is successful.
if ($extraction_failed) {
    Exit
} else {
    Invoke-WebRequest -Method POST -Uri $end_of_upload_api
}
